@@ -8,7 +8,7 @@ package machine
 var (
 	UART0 = hardwareUART0
 	UART1 = hardwareUART1
-	SPI0  = SPI{0}
-	SPI1  = SPI{1}
+	SPI0  = &SPI{0}
+	SPI1  = &SPI{1}
 	I2C0  = &I2C{0}
 )
