@@ -1,20 +1,9 @@
-// +build wasm,!wasi
+//go:build wasm && !wasip1
 
 package runtime
 
-import "unsafe"
-
 type timeUnit float64 // time in milliseconds, just like Date.now() in JavaScript
 
-//export _start
-func _start() {
-	// These need to be initialized early so that the heap can be initialized.
-	heapStart = uintptr(unsafe.Pointer(&heapStartSymbol))
-	heapEnd = uintptr(wasm_memory_size(0) * wasmPageSize)
-
-	run()
-}
-
 var handleEvent func()
 
 //go:linkname setEventHandler syscall/js.setEventHandler
@@ -22,19 +11,6 @@ func setEventHandler(fn func()) {
 	handleEvent = fn
 }
 
-//export resume
-func resume() {
-	go func() {
-		handleEvent()
-	}()
-	scheduler()
-}
-
-//export go_scheduler
-func go_scheduler() {
-	scheduler()
-}
-
 func ticksToNanoseconds(ticks timeUnit) int64 {
 	// The JavaScript API works in float64 milliseconds, so convert to
 	// nanoseconds first before converting to a timeUnit (which is a float64),
@@ -50,8 +26,9 @@ func nanosecondsToTicks(ns int64) timeUnit {
 
 // This function is called by the scheduler.
 // Schedule a call to runtime.scheduler, do not actually sleep.
-//export runtime.sleepTicks
+//
+//go:wasmimport gojs runtime.sleepTicks
 func sleepTicks(d timeUnit)
 
-//export runtime.ticks
+//go:wasmimport gojs runtime.ticks
 func ticks() timeUnit
